/*! \file BibblePlugin.h
	\brief Bibble Plugin SDK Interface 
*/


#ifndef BIBBLE5PLUGIN_H
#define BIBBLE5PLUGIN_H

#include <stdio.h>

//! The current version of the interface
/*! The interface version is only incremented when changes are made
	that make previous plug-ins in compatible
*/
#define BIBBLE5PLUGIN_API_VERSION 7

//! Name of the dynamic property in the Qt UI file for the plug-in name
/*!	The value of this string property identifies the UI file as belonging
	to the plug-in. Bibble uses the name to find the correct callback function
	for the plug-in 
*/
#define BIBBLE_PLUGIN_NAME	"BIBBLE_PLUGIN_NAME"
//! Name of the dynamic property in the Qt UI file for the plug-in group
/*!	The value of this string property identifies the controls in the UI file 
	as belonging to the plug-in group. Bibble uses the group to find the 
	correct options for the controls. Option IDs must be unique within a
	particular plug-in group.  
*/
#define BIBBLE_PLUGIN_GROUP "BIBBLE_PLUGIN_GROUP"

#ifdef WIN32
 #define BBOOL	char

 #ifdef  _WIN64
  typedef unsigned __int64    BHANDLE;
 #else
  typedef _W64 unsigned int   BHANDLE;
 #endif

#else
#include <stdint.h>
 #define BBOOL	int
 typedef intptr_t BHANDLE;
#endif

//! A rectangle
/*! BRects are defined by X1,Y1<->-X2,Y2 inclusive
*/
struct BRect
{
	int x1,y1;
	int x2,y2;
};

//! A width and height in one tidy package
struct BSize
 {
	int w,h;
 };

//! 2D co-ordinates
struct BPoint
 {
	int x,y;
 };

//!\defgroup PipeStages Plugin Pipe Stage Names
//!@{
//! Pipe stage names
//! Plugins are run in the "PLUGIN" stages which happen before their respective Bibble stages

//! Noise Ninja - any plugins before this stage could invalidate the
//! pre-generated profiles and negatively impact Noise Ninja performance
#define PIPE_PLUGIN_NOISENINJA		30
//! Removes gamma to make rendered images linear (not called for RAW)
#define PIPE_PLUGIN_DEGAMMA			35
//! Lens correction vignette
#define PIPE_PLUGIN_POSVIGNETTE		40	
//! Highlight shaping for HR, exposure, contrast
#define PIPE_PLUGIN_EXPOSURE		50	
//! Fill light
#define PIPE_PLUGIN_FILL			100 
//! White balance adjustment (data is always white balanced in the pipe)
#define PIPE_PLUGIN_WB				150 
//! Adds back gamma to rendered images (not called for RAW)
#define PIPE_PLUGIN_PRECOLOR		200
//! Converts raw images from 14 bit to 16 bit (not called for rendered images)
#define PIPE_PLUGIN_14TO16			225
//! Converts from RGB to Lab
#define PIPE_PLUGIN_COLORMANAGE		250
//! Bibble sharpening
#define PIPE_PLUGIN_SHARPEN			300
//! Lab to RGB
#define PIPE_PLUGIN_COLORMANAGE2	350
//! Overall Hue/Saturation/Vibrance 
#define PIPE_PLUGIN_COLOR			510 
//! Perfectly clear
#define PIPE_PLUGIN_PERFECTLYCLEAR	450 
//! Selective color
#define PIPE_PLUGIN_SELECTIVECOLOR	500
//! Auto-level
#define PIPE_PLUGIN_AUTOLEVEL		600
//! Artistic vignette (darkening)
#define PIPE_PLUGIN_NEGVIGNETTE		610
//! Curves
#define PIPE_PLUGIN_CURVES		550
//! Stage for plug-ins that want to run after curves
#define PIPE_PLUGIN_FINAL		650
//!@}

//!\defgroup OptionFlags Option Flags
//!@{
//! Flags that can be added to options

//! Don't restart the processing when this option changes
#define OPTION_DONT_RESTART		4
//! Setting doesn't get saved with the version
#define OPTION_DOESNOTPERSIST	8
//! Do not do dependency checks on this option
#define OPTION_NO_DEPENDENCY	512
//! Option only appears in the background layer and can't be edited if other layers are selected
#define OPTION_NOLAYERS			2048
//! Restart processing when this option changes
#define OPTION_RESTART			4096	
//! Option only appears in the background layer, but should always be able to be edited, for example keywords
#define OPTION_GLOBAL			8192	
//! These options should not be displayed in the GUI
#define OPTION_NO_DISPLAY		16384	
//! Changing these options causes the source image to be reprocessed
#define OPTION_RESTART_SOURCE	32768	
//!@}

typedef BHANDLE  BPluginId;
typedef BHANDLE  BStringHandle;
typedef BHANDLE  BTileHandle;
typedef BHANDLE  BImageHandle;
typedef BHANDLE  BHistogramHandle;
typedef BHANDLE  BOptionListHandle;
typedef BHANDLE  BMemoryHandle;
typedef BHANDLE  BComboBoxHandle;
typedef BHANDLE  BSettingsWidgetHandle;

//!\defgroup RequiredCallbacks Required Callback Functions
//!@{
//! All plug-ins must export these functions

//! This function gets called when Bibble loads the plug-in
typedef int	(*RegisterPluginPtr)(struct BibblePluginData *data);
//! This function gets called when Bibble unload the plug-in
typedef int	(*UnRegisterPluginPtr)(struct BibblePluginData *data);
//! This function gets called before attempting to load the plug-in to verify it is compatible
typedef int	(*getVersionPtr)(void);
//! Bibble calls the function to ask if the plug-in is going to run and whether to copy the previous step's output to prev
typedef bool (*PluginRunCheckPtr)(struct BibblePluginData *data, struct BPluginPipeData *pipeData, BBOOL &needPrev );

//! Bibble calls this function once to add your options to the option table

//! Once the table is finalized, no more options can be added
typedef int (*PluginBuildDefaultsPtr)(struct BibblePluginData *data, struct BAddOptionFunctions *addFunctions);
//!@}

//! General utility functions
struct BUtilityFunctions
{
	//! Display a dialog containing text that is dismissed with an OK button
	void			(*m_msgbox)(const char *caption, const char *text);

	//! Print a debug message that is also written into bibbledebug.txt

	//! This function is not available in the register plug-in function
	void			(*m_trace)(const char *a);

	//! Display the standard file open dialog to get a file path from the user

	//! If the return value is NULL, the dialog was cancelled
	//! You should free the BStringHandle returned by this function when you're finished with it
	//! The filter is a string of the format "Filter1 Description (*.ext1a, *.ext1a) ;; Filter2 Description (*.ext2a)" etc.
	//! For example, "Images (*.png *.xpm *.jpg);;Text files (*.txt);;XML files (*.xml)"
	BStringHandle	(*m_getOpenFilename)( const char *caption, const char *dir, const char *filter);
	unsigned char pad[256];	
};

//! Functions to manipulate strings
struct BStringFunctions
{
	//! Allocate an empty string
	BStringHandle	(*m_alloc)();
	//! Allocate a string and set its contents to ascii string a
	BStringHandle	(*m_allocFromAscii)(const char *a);
	//! Allocate a string and set its contents to unicode string u with length l
	BStringHandle	(*m_allocFromUnicode)(unsigned short *u,int l);
	//! Free a previously allocated string 
	void			(*m_free)(BStringHandle a);
	//! Get the length of string a
	int				(*m_length)(BStringHandle a);
	//! Copy the contents of string s into d 
	void			(*m_copy)(BStringHandle d,BStringHandle s);
	//! Allocate a duplicate of string s (you should free the duplicate when you're finished with it)
	BStringHandle	(*m_dup)(BStringHandle s);
	//! Get the contents of string s as ASCII
	char*			(*m_ascii)(BStringHandle s);
	//! Get the contents of string s as Unicode
	unsigned short* (*m_unicode)(BStringHandle s);
	//! Copy the ASCII contents of a into string s
	void			(*m_fromAscii)(BStringHandle s, const char *a);
	//! Copy the Unicode contents of u with length l into string s
	void			(*m_fromUnicode)(BStringHandle s,unsigned short *u,int l);
	//! Compare string s to string a (case sensitive) and return 0 if they are identical
	int				(*m_compare)(BStringHandle s, BStringHandle a);
	//! Compare string s to string a (case sensitive) and return 0 if they are identical
	int				(*m_compareCString)(BStringHandle s, const char * a);
	//! Concatenate string d and string add
	void			(*m_concat)(BStringHandle d, BStringHandle add);
	//! Concatenate string d and string add
	void			(*m_concatCString)(BStringHandle d, const char * add);
	//! Get the contents of string d as UTF-16
	unsigned short *(*m_ucs2)(BStringHandle d);

	//! Translate text using context and return the result in string s, which has already been allocated
	void			(*m_translate)(BStringHandle s, const char* context, const char *text);

	unsigned char	pad[256];
};

//! Functions to access image information
struct BImageFunctions
{
	//! Get a pointer to the begining of scan line i for image b. The top of the image is scanline 0
	unsigned char *		(*m_scanline)(BImageHandle b,int i);
	//! Get a pointer to the begining of scan line i for image b. The top of the image is scanline 0
	unsigned short *	(*m_scanline16)(BImageHandle b,int i);
	//! Returns a non-zero value if the image is planer (not interleaved)

	//! Planer means all red values are together, followed by all green values, followed by all the blue values
	int					(*m_planer)(BImageHandle b);
	//! The number of bytes in one row of image b
	int					(*m_rowBytes)(BImageHandle b);
	//! The number of words (2 bytes) in one row of image b
	int					(*m_rowWords)(BImageHandle b);
	//! The number of bytes between one color plane and the next 
	int					(*m_planeBytes)(BImageHandle b);
	//! The number of words (2 bytes) between one color plane and the next
	int					(*m_planeWords)(BImageHandle b);
	//! The size of the padding on each side of the image
	int					(*m_padWidth)(BImageHandle b);
	//! The number of lines of padding at the top and bottom of the image
	int					(*m_padLines)(BImageHandle b);
	//! The number of color channels in the image
	int					(*m_numChannels)(BImageHandle b);
	//! The number of bytes used to represent one color channel (1 for 8 bpp and 2 for 16 bpp)
	int					(*m_bytesPerChannel)(BImageHandle b);
	//! The number of bytes between each pixel
	int					(*m_nextPixel)(BImageHandle b);
	//! The width of the image in pixels
	int					(*m_width)(BImageHandle b);
	//! The height of the image in pixels
	int					(*m_height)(BImageHandle b);
	unsigned			char	pad[256];
};

//! Functions to access information from a tile
struct BTileFunctions
{
	//! A pointer to the image contained in the tile
	BImageHandle (*m_image)(BTileHandle tile);
	//! The x location of this tile in the image, with the left side as pixel 0
	int (*m_x)(BTileHandle tile);
	//! The y location of this tile in the image, with the top as line 0
	int (*m_y)(BTileHandle tile);

	//! The width and height of the real data in this tile

	//! This will be the same as the image width and height
	//! unless the tile is along the right or bottom of the image
	//! and only partially filled
	BSize (*m_activeSize)(BTileHandle tile);
	unsigned char pad[256];
};

//! Function to get and set options

//! A return value of false for the getOption functions is not an error... it means that the default 
//! value was returned because the list didn't contain a value for the requested option. 
//! All options have an "owner". The plug-in ID assigned by Bibble when the plug-in is registered, 
//! and passed in the BibblePluginData struct for each call is the 
//! owner of all options registered by this plug-in. To access Bibble options, use owner 0
struct BOptionsFunctions
{
	//! Returns true if the option list ol has optid owned by the plug-in owner explicitly set to a value 
	bool		(*m_optionListContains)(BOptionListHandle ol, int optid, BPluginId owner );
	//! Set boolean option optid registered by owner in list ol to val
	void		(*m_setOptionBool)(BOptionListHandle ol, int optid, BPluginId owner, bool val);
	//! Get the value of boolean option optid registered by owner in list ol in val
	bool		(*m_getOptionBool)(BOptionListHandle ol, int optid, BPluginId owner, bool &val);
	//! Set string option optid registered by owner in list ol to val
	void		(*m_setOptionString)(BOptionListHandle ol, int optid, BPluginId owner, BStringHandle val);
	//! Get the value of string option optid registered by owner in list ol in val
	bool		(*m_getOptionString)(BOptionListHandle ol, int optid, BPluginId owner, BStringHandle val);
	//! Set double option optid registered by owner in list ol to val
	void		(*m_setOptionDouble)(BOptionListHandle ol, int optid, BPluginId owner, double val);
	//! Get the value of double option optid registered by owner in list ol in val
	bool		(*m_getOptionDouble)(BOptionListHandle ol, int optid, BPluginId owner, double &val);
	//! Set integer option optid registered by owner in list ol to val
	void		(*m_setOptionInt)(BOptionListHandle ol, int optid, BPluginId owner, int val);
	//! Get the value of integer option optid registered by owner in list ol in val
	bool		(*m_getOptionInt)(BOptionListHandle ol, int optid, BPluginId owner, int &val);
	//! Get the id of option named optName and registered by owner
	int			(*m_getOptionIDByName)(BStringHandle optName, BPluginId owner );
	unsigned char pad[256];
};

//! Functions to add options to Bibble's option tables in the buildDefaults function
struct BAddOptionFunctions 
{
	//! Register boolean option 
	int (*m_addBoolOption)	(BPluginId owner, int optid, const char *name, const char *dispname, const char *group, const char *desc, BBOOL value, int flags);
	//! Register an integer option
	int (*m_addIntOption)	(BPluginId owner, int optid, const char *name, const char *dispname, const char *group, const char *desc, int value, int flags);
	//! Register a double option
	int (*m_addDoubleOption)(BPluginId owner, int optid, const char *name, const char *dispname, const char *group, const char *desc, double value, int flags);
	//! Register a string option
	int (*m_addStringOption)(BPluginId owner, int optid, const char *name, const char *dispname, const char *group, const char *desc, const char *value, int flags);

	unsigned char	pad[256]; 
};

//! Functions to allocate and free memory
struct BMemoryFunctions
{
	BMemoryHandle	(*m_alloc)(int size);
	void			(*m_free) (BMemoryHandle s);
	void*			(*m_ptr) (BMemoryHandle s);
	unsigned char	pad[256]; 
};

//! Functions to manipulate widgets in your UI
struct BSettingsWidgetFunctions
{
	//! Get a handle to a drop down list in widget s named name
	BComboBoxHandle (*m_getCombo)(BSettingsWidgetHandle s,const char *name);
	//! Get the name of widget s
	void  (*m_name)(BSettingsWidgetHandle s, BStringHandle name);
	//! Set the enabled state of widget named name in widget sh to enable (true or false)
	void (*m_setEnabled)( BSettingsWidgetHandle sh, const char *name, bool enable );
	//! Set the visible state of widget named name in widget sh to enable (true or false)
	void (*m_setVisible)( BSettingsWidgetHandle sh, const char *name, bool visible );
	//! Set the text of the QLabel named name in widget sh to text
	void (*m_setLabelText)( BSettingsWidgetHandle sh, const char *name, const char *text);

	unsigned char pad[256];	
};

//! Functions to manipulate drop down lists
struct BComboBoxFunctions
{
	//! Get a count of the items in combobox cb
	int		(*m_count)(BComboBoxHandle cb);
	//! Remove all the items in combobox cb
	void	(*m_clear)(BComboBoxHandle cb);
	//! Append  item text in combobox cb after position pos (-1 means at the beginning)
	void	(*m_append)(BComboBoxHandle cb, const char *text,int pos);
	//! Set the selected position to pos for combobox cb
	void	(*m_setPos)(BComboBoxHandle cb, int pos);
	//! Get the selected position for combobox cb
	int		(*m_getPos)(BComboBoxHandle cb);
	//! Get the selected text for combobox cb in string txt, which has already been allocated
	void	(*m_getText)(BComboBoxHandle cb, BStringHandle txt);
	//! Get the text for position i in combobox cb in string txt, which has already been allocated
	void	(*m_getTextItem)(BComboBoxHandle cb,BStringHandle txt, int i);
	//! Set the text of the currently selected item to txt
	void	(*m_setText)(BComboBoxHandle cb, BStringHandle txt);

	unsigned char pad[256];
};

//! Information about the state of the processing pipe 

//! Not all parts of this structure are set for all function types where it is passed, so check for NULL values
struct BPluginPipeData
{
	//! The tile being processed
	BTileHandle m_tile;				
	//! Alpha mask for the current layer
	BImageHandle m_alpha;			
	//! A temporary buffer for holding processed image before blending into the tile image
	BImageHandle m_dest;			
	//! The image at the beginning of the state
	BImageHandle m_prev;			
	//! Options list for the main layer
	BOptionListHandle m_mainOptions;
	//! Options list for the current layer
	BOptionListHandle m_options;
	//! The index of the current layer
	int m_layerNum;			
	//! Number of layers in the current settings
	int m_numLayers;	
	//! The pipe stage the function was registered for; useful if you want to map the same function to more than one stage
	int m_currentStage;	
	//! Current mip level
	int m_currentMip;	
	//! Set this flag if Bibble should blend m_dest into the current image
	bool m_blend;
	//! The width and height of the full sized image
	BSize m_mipZeroSize;

	unsigned char pad[256];
};

//! The signaure of the function called to process a tile
typedef int	(*PipeFunctionPtr )(struct BibblePluginData *data, struct BPluginPipeData *pipeData);

//! SettingsWidgetAboutToChangePtr is called whenever the settings for the interactive image are about change

//! This won't be called for any background processing, like batch queues
//! Everything except for m_widget and m_optionName is set in OptionData
//! Any options you change should be set by the time SettingsChangedPtr is called
typedef int	(*SettingsWidgetAboutToChangePtr )(struct BibblePluginData *data,struct OptionData *od);

//! SettingsWidgetDoDependanciesPtr is called whenever a widget changes the settings for the interactive image

//! This gives you a chance to handle interactions in your widget like enabling your plugin when a control changes
//! It is not called if settings are changed programatically, and it should not prevent SettingsChangedPtr
//! from being called
//! Everything in the OptionData structure should be populated
typedef int	(*SettingsWidgetDoDependanciesPtr )(struct BibblePluginData *data,struct OptionData *od);

//! SettingsChangedPtr is called whenever the settings for the interactive image change

//! This won't be called for any background processing, like batch queues
//! Everything except for m_widget and m_optionName is set in OptionData
typedef int	(*SettingsChangedPtr )(struct BibblePluginData *data,struct OptionData *od);

//! This function is called when the plug-ins UI is created
typedef int	(*OptionsInitPtr )(struct BibblePluginData *data, BSettingsWidgetHandle widget);	

//! The signature of the function called when a plug-in button is pressed
typedef int	(*ButtonCallBackPtr)(struct BibblePluginData *data, struct OptionData *od); 

//! HotnessChangedPtr is called whenever an image is set as the interactive image

//! This won't be called for any background processing, or if an image is just added to a selection
//! Only the m_mainList, m_currentList, and m_currentLayer are set in OptionData
//! This can also happen when the interactive image is deselected. In that case the option data fields are all NULL
typedef int	(*HotnessChangedPtr )(struct BibblePluginData *data,struct OptionData *od);		

//! Function to register plug-in functions with Bibble
struct BRegistrationFunctions
{
	//! Register a function that gets called at pipe state stage
	void	(*m_registerPipeRoutines)(BPluginId, PipeFunctionPtr, int stage, int priority);	
	//! Register the function that gets called when settings are about to change
	void	(*m_registerSettingsWidgetOptionAboutToChange)(BPluginId,SettingsWidgetAboutToChangePtr);
	//! Reigster the function that gets called after a control is changed by the user
	void	(*m_registerSettingsWidgetDoDependancies)(BPluginId,SettingsWidgetDoDependanciesPtr);
	//! Register the function to call when the plug-in's UI is initialized
	void	(*m_registerOptionsInit)(BPluginId,OptionsInitPtr);	
	//! Register the function that gets called when the active image changes
	void	(*m_registerHotnessChanged)(BPluginId, HotnessChangedPtr);
	//! Register a function that gets called when a button named name is clicked
	int		(*m_registerButtonCallback)(BPluginId,ButtonCallBackPtr,const char *name);

	//! Register the function that gets called when settings have changed
	void	(*m_registerSettingsChanged) (BPluginId, SettingsChangedPtr);

	//! Use this registration callback for buttons that should cause a settings change for boolean option optid
	int		(*m_registerSettingsButtonCallback)(BPluginId,ButtonCallBackPtr,const char *name, int optid);

	unsigned char pad[256-4];			
};

//! Option List data
struct OptionData
{
	//! The currently selected layer
	int m_currentLayer;
	//! Options that have changed
	BOptionListHandle m_changeList;	
	//! Options for the current layer
	BOptionListHandle m_currentList;
	//! Options for the main layer
	BOptionListHandle m_mainList;
	//! The widget that received the settings change
	BSettingsWidgetHandle m_widget;
	//! Name of the option that's changing
	BStringHandle m_optionName;			

	unsigned char pad[256];	
};

//! Global data for the plug-in and pointers to the API functions
struct BibblePluginData
{
	//! The ID assigned by Bibble to your plugin
	BPluginId	m_plugin;
	
	int	m_version;
	int	m_size;

	//! Unique name used to identify this plugin
	BStringHandle	m_pluginName;	
	//! Unique name used to group this plugins with others (developer name would be a good choice)
	//! You are only guaranteed a particular priority order if the plugins belong to the same group
	BStringHandle	m_groupName;	

	//! Location of the user specific file used to store settings etc
	BStringHandle	m_userHome;		
	//! "Common" location used to store libraries profiles etc /supportfiles
	BStringHandle	m_systemHome;	

	//! Global data for the plug-in. Allocated and freed by the plug-in
	void *m_userdata;				

	//!@{
	//! Pointers for functions provided by the API
	struct BUtilityFunctions m_buf;
	struct BStringFunctions	m_bsf;
	struct BImageFunctions	m_bif;
	struct BTileFunctions m_btf;
	struct BRegistrationFunctions m_brf;
	struct BMemoryFunctions m_bmf;
	struct BOptionsFunctions m_bof;
	struct BComboBoxFunctions m_bcb;
	struct BSettingsWidgetFunctions m_bsw;
	//!@}
	unsigned char pad[256];			
};

// Macros
#ifndef BIBBLEPLUGINNOMACROS
// Utility
#define BDebug(a)		(bibbleData->m_buf.m_trace(a))
#define BMsgBox(a,b)	(bibbleData->m_buf.m_msgbox((a),(b)))
#define BGetOpenFileName(a,b,c) (bibbleData->m_buf.m_getOpenFilename( (a), (b), (c) ))

// String
#define BStringAlloc() (bibbleData->m_bsf.m_alloc())
#define BStringAllocFromAscii(a) (bibbleData->m_bsf.m_allocFromAscii(a))
#define BStringAllocFromUnicode(u,l) (bibbleData->m_bsf.m_allocFromUnicode((u),(l)))
#define BStringFree(a) (bibbleData->m_bsf.m_free(a))
#define BStringLength(a) (bibbleData->m_bsf.m_length(a))
#define BStringCopy(d,s) (bibbleData->m_bsf.m_copy((d),(s)))
#define BStringDup(s) (bibbleData->m_bsf.m_dup((s)))
#define BStringUnicode(s) (bibbleData->m_bsf.m_unicode(s))
#define BStringAscii(s) (bibbleData->m_bsf.m_ascii(s))
#define BStringFromUnicode(s,u) (bibbleData->m_bsf.m_fromUnicode((s),(u)))
#define BStringFromAscii(s,a) (bibbleData->m_bsf.m_fromAscii((s),(a)))
#define BStringCompare(s,a) (bibbleData->m_bsf.m_compare((s),(a)  ))
#define BStringCompareCString(s,a) (bibbleData->m_bsf.m_compareCString((s),(a)  ))
#define BStringConcat(s,a) (bibbleData->m_bsf.m_concat((s),(a)  ))
#define BStringConcatCString(s,a) (bibbleData->m_bsf.m_concatCString((s),(a)  ))
#define BStringUCS2(s) (bibbleData->m_bsf.m_ucs2((s)))		
#define BStringTranslate(s, c, t) (bibbleData->m_bsf.m_translate( (s), (c), (t) ))

// Image functions
#define BImageScanLine(s,a) (bibbleData->m_bif.m_scanline( (s),(a) ) )
#define BImageScanLine16(s,a) (bibbleData->m_bif.m_scanline16( (s),(a) ) )
	    
#define BImagePlanar(s) (bibbleData->m_bif.m_planer( (s) ) )
#define BImageRowBytes(s) (bibbleData->m_bif.m_rowBytes( (s) ) )
#define BImageRowWords(s) (bibbleData->m_bif.m_rowWords( (s) ) )
#define BImagePlaneBytes(s) (bibbleData->m_bif.m_planeBytes( (s) ) )
#define BImagePlaneWords(s) (bibbleData->m_bif.m_planeWords( (s) ) )
#define BImagePadWidth(s) (bibbleData->m_bif.m_padWidth( (s) ) )
#define BImagePadLines(s) (bibbleData->m_bif.m_padLines( (s) ) )
#define BImageNumChannels(s) (bibbleData->m_bif.m_numChannels( (s) ) )
#define BImageBytesPerChannel(s) (bibbleData->m_bif.m_bytesPerChannel( (s) ) )
#define BImageWidth(s) (bibbleData->m_bif.m_width( (s) ) )
#define BImageHeight(s) (bibbleData->m_bif.m_height( (s) ) )
#define BImageNextPixel(s) (bibbleData->m_bif.m_nextPixel( (s) ) )

// Tile functions
#define BTileImage(t) (bibbleData->m_btf.m_image( (t) ))
#define BTileX(t) (bibbleData->m_btf.m_x( (t) ))
#define BTileY(t) (bibbleData->m_btf.m_y( (t) ))
#define BTileActiveSize(t) (bibbleData->m_btf.m_activeSize( (t) ))

// Options
#define BOptionListContains(l,o,p) (bibbleData->m_bof.m_optionListContains((l),(o),(p)))
#define BSetOptionBool(l,o,p,v) (bibbleData->m_bof.m_setOptionBool( (l), (o), (p), (v)))
#define BGetOptionBool(l,o,p,v) (bibbleData->m_bof.m_getOptionBool( (l), (o), (p), (v)))
#define BSetOptionString(l,o,p,v) (bibbleData->m_bof.m_setOptionString( (l), (o), (p), (v)))
#define BGetOptionString(l,o,p,v) (bibbleData->m_bof.m_getOptionString( (l), (o), (p), (v)))
#define BSetOptionDouble(l,o,p,v) (bibbleData->m_bof.m_setOptionDouble( (l), (o), (p), (v)))
#define BGetOptionDouble(l,o,p,v) (bibbleData->m_bof.m_getOptionDouble( (l), (o), (p), (v)))
#define BSetOptionInt(l,o,p,v) (bibbleData->m_bof.m_setOptionInt( (l), (o), (p), (v)))
#define BGetOptionInt(l,o,p,v) (bibbleData->m_bof.m_getOptionInt( (l), (o), (p), (v)))
#define BGetOptionIDByName(o, p) (bibbleData->m_bof.m_getOptionIDByName( (o), (p) ))

// BSettingsWidget
#define BSettingsWidgetGetCombo( w, n ) (bibbleData->m_bsw.m_getCombo( (w), (n) ) )
#define BSettingsWidgetName( w, s ) (bibbleData->m_bsw.m_name( (w), (s) ) )
#define BSettingsWidgetSetEnabled( w, n, b ) (bibbleData->m_bsw.m_setEnabled( (w), (n), (b) ) )
#define BSettingsWidgetSetVisible( w, n, b ) (bibbleData->m_bsw.m_setVisible( (w), (n), (b) ) )
#define BSettingsWidgetSetLabel( w, n, t ) (bibbleData->m_bsw.m_setLabelText( (w), (n), (t) ) )

// BComboBox
#define BComboBoxClear(a) (bibbleData->m_bcb.m_clear( (a) ) )
#define BComboBoxCount(a) (bibbleData->m_bcb.m_count( (a) ) )
#define BComboBoxSetPos(a,b) (bibbleData->m_bcb.m_setPos( (a), (b) ))
#define BComboBoxGetPos(a) (bibbleData->m_bcb.m_getPos( (a) ))
#define BComboBoxGetBString(a,b) (bibbleData->m_bcb.m_getText( (a) , (b)  ) )
#define BComboBoxGetBStringItem(a,b,c) (bibbleData->m_bcb.m_getTextItem( (a) , (b) ,(c) ) )
#define BComboBoxSetBString(a,b) (bibbleData->m_bcb.m_setText( (a) , (b)  ) )
#define BComboBoxAppend(a,b,c) (bibbleData->m_bcb.m_append( (a), (b), (c)) )

#endif // ifndef BIBBLEPLUGINNOMACROS

#endif // #ifndef BIBBLEPLUGIN_H
