/*

Copyright (c) 2010 Thomas Baruchel

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

*/

#include "NegativePositive.h"

#include <stdlib.h>

BDebugTraceFunction BTestDebug;

int NpConvert(struct BibblePluginData *bibbleData, struct BPluginPipeData *pipeData) {
	int rc = 0;

	// Grab the image data and info
	BImageHandle bimg  = BTileImage(pipeData->m_tile);

	// If we don't have an image, we can't do anything
	if ( bimg == 0)
	{
		BMSG( "NpConvert() failed - NULL image");
		return -1;
	}

	pipeData->m_blend = false; 

        int bpc;
        double gamma;
	bibbleData->m_bof.m_getOptionInt( pipeData->m_options,
          NP_BPC, bibbleData->m_plugin, bpc);
	bibbleData->m_bof.m_getOptionDouble( pipeData->m_options,
          NP_GAMMA, bibbleData->m_plugin, gamma);
        double maxval[9] = {
          255, 511, 1023, 2047, 4095, 8191, 16383, 32767, 65535
        };
        double igamma = ((double)1.0)/gamma;
        unsigned short lut[65536];
        for(int i=0;i<=maxval[bpc];i++) {
          lut[i] = (unsigned short)
            (maxval[bpc]*pow(1.0-pow(((double)i)/maxval[bpc],igamma),gamma)
              + ((double) 0.5));
        }

	int w,h,nChan,pw,rw,padw,padh,np;

	w = BImageWidth(bimg);
	h = BImageHeight(bimg);
	nChan = BImageNumChannels(bimg);
	pw = BImagePlaneWords(bimg);	// plane words - how far to get to the next plane
	rw = BImageRowWords(bimg);		// number of unsigned shorts in a row (includes padding)
	padw = bibbleData->m_bif.m_padWidth( bimg );
	padh = BImagePadLines(bimg);
	np = BImageNextPixel(bimg);		// increment to get to the next pixel


	if ( nChan != 3 )
	{
		BMSG("NpConvert() - Not a 3 channel image\n");
		// We only handle RGB data
		return -1;
	}

	unsigned short *pIn, *pEnd;

	for( int y = -padh; y < h+padh; ++y ) {
		pIn = BImageScanLine16( bimg, 0 ) + y*rw - padw*np;
		pEnd = pIn + (w+padw*2)*np;
		while( pIn < pEnd ) {
		  *(pIn) = lut[*(pIn)];
		  *(pIn+pw) = lut[*(pIn+pw)];
		  *(pIn+2*pw) = lut[*(pIn+2*pw)];
		  pIn += np;
		}
	}
	return rc;
}

// You may name the function anything you like, but if you want to use the macros to
// call the Bibble functions, you must keep the parameter names the same
// typedef int	(*OptionsInitPtr )(struct BibblePluginData *bibbleData, struct OptionData *od);

int initializeOptions(struct  BibblePluginData *bibbleData, BSettingsWidgetHandle sw)
{
	// This function is called when the plugin loads
	int rc = 0;

	BComboBoxHandle cb = BSettingsWidgetGetCombo( sw,
          "negativepositive_bpc_combo" );
	if (cb==0)
	{
		BMSG("Bathsheba Plugin couldn't find negativepositive_bpc ComboBox");
		return -1;
	}

	// A value of -1 means prepend
	BComboBoxClear(cb);

	BComboBoxAppend(cb, "8 bpc", 0 );
	BComboBoxAppend(cb, "9 bpc", 1 );
	BComboBoxAppend(cb, "10 bpc", 2 );
	BComboBoxAppend(cb, "11 bpc", 3 );
	BComboBoxAppend(cb, "12 bpc", 4 );
	BComboBoxAppend(cb, "13 bpc", 5 );
	BComboBoxAppend(cb, "14 bpc", 6 );
	BComboBoxAppend(cb, "15 bpc", 7 );
	BComboBoxAppend(cb, "16 bpc", 8 );

        // Default
	BComboBoxSetPos(cb, 4);

	return rc;
}
int doDependancies( struct BibblePluginData *bibbleData, struct OptionData *od) {
	int rc = 0;

	// Get a handle to our widget... if we can't find it, we can't initialize our options
        /*
	BSettingsWidgetHandle wh = od->m_widget;
	if ( NULL == wh ) return -1;
        */

#ifdef _DEBUG
	BMSG("Negative to Positive - Do Dependancies");
#endif
	if ((BOptionListContains(od->m_changeList,NP_BPC,bibbleData->m_plugin)) || (BOptionListContains(od->m_changeList,NP_GAMMA,bibbleData->m_plugin)))
	{
		BSetOptionBool(od->m_changeList,NP_ENABLED,bibbleData->m_plugin,true);
	}

	if ( BOptionListContains( od->m_changeList, NP_BPC, bibbleData->m_plugin) )
	{
	  int bpc;
	  BSettingsWidgetHandle wh = od->m_widget;
	  BComboBoxHandle cb = BSettingsWidgetGetCombo( wh,
            "negativepositive_bpc_combo" );	
		BGetOptionInt( od->m_changeList, NP_BPC, bibbleData->m_plugin, bpc);
		BComboBoxSetPos(cb, bpc);
	}

	return rc;
}

// This function is a registered button callback
int NpHandleButton( struct BibblePluginData *bibbleData, struct OptionData *od ) {
	int rc = 0;

#ifdef _DEBUG
	BMSG("Negative to Positive - NpHandleButton()" );
#endif

	BMsgBox( "Negative to Positive", "Version 1.1\n\nWritten by Thomas Baruchel\nhttp://baruchel.free.fr/bibble/\n\nThis plugin is an open-source software released under\nthe terms of the MIT license.\nhttp://www.opensource.org/licenses/mit-license.php\n\nIt is not only free as in beer, but also free as in freedom.\nYou may use it, you may share it, you may change it, you\nmay even sell it." );
	return rc;
}

__declspec(dllexport) int RegisterPlugin(struct BibblePluginData *bibbleData)
{
	// This function is called by Bibble during loading
	// Plug-ins should register the functions Bibble needs to call,
	// check that they are compatible with the version of Bibble attempting to load them,
	// add their settings to the factory defaults list for the Bibble option system,
	// and allocate and initialize any global data or resources
	int rc = 0;

	BTestDebug = (BDebugTraceFunction)(bibbleData->m_buf.m_trace);

	// Set the API version we were compiled under
	// This is defined in BibblePlugin.h
	// Set the API version we were compiled under
	// This is defined in BibblePlugin.h
	bibbleData->m_version = BIBBLE5PLUGIN_API_VERSION;
	bibbleData->m_bsf.m_fromAscii(bibbleData->m_groupName, NP_PLUGINGROUP);
	bibbleData->m_bsf.m_fromAscii(bibbleData->m_pluginName, NP_PLUGINNAME);

	bibbleData->m_brf.m_registerPipeRoutines(bibbleData->m_plugin, NpConvert, PIPE_PLUGIN_EXPOSURE, 200);
	bibbleData->m_brf.m_registerOptionsInit(bibbleData->m_plugin, initializeOptions);

	// Register my button callback
	bibbleData->m_brf.m_registerButtonCallback(bibbleData->m_plugin, NpHandleButton, NP_BUTTONNAME);

	bibbleData->m_brf.m_registerSettingsWidgetDoDependancies(bibbleData->m_plugin, doDependancies );

	BMSG("Negative to Positive plug-in loaded");
	return rc;
}

__declspec(dllexport) int unRegisterPlugin(struct BibblePluginData *bibbleData)
{
	// This function is called by Bibble right before the program exits
	// Plug-ins should free any global memory and resources here
	int rc = 0;

	return rc;
}

__declspec(dllexport) int getVersion( void )
{
	return BIBBLE5PLUGIN_API_VERSION;
}

__declspec(dllexport) int buildDefaults(struct BibblePluginData *bibbleData, struct BAddOptionFunctions *baf)
{
	int rc = 0;

	baf->m_addBoolOption( bibbleData->m_plugin, NP_ENABLED, "negativepositive_enabled", "Enable Negative to Positive", NP_GROUP, "Enable Negative to Positive.", false, OPTION_RESTART|OPTION_NO_DEPENDENCY);
	baf->m_addIntOption( bibbleData->m_plugin, NP_BPC, "negativepositive_bpc", "Bits per channel", NP_GROUP, "Set the number of bits per channel\n(according to your capture engine).\nYou should look for the right value\nand put it into your default settings.", 4, OPTION_RESTART);
	baf->m_addDoubleOption( bibbleData->m_plugin, NP_GAMMA, "negativepositive_gamma", "Gamma", NP_GROUP, "Set the gamma value.\nThe best should be between 1.8 and 2.2\n(no gamma correction with 1.0).", 1.80, OPTION_RESTART);
	return rc;
}

__declspec(dllexport) bool runCheck(struct BibblePluginData *bibbleData, struct BPluginPipeData *pipeData, BBOOL &needPrev )
{
	bool enabled = false;
	needPrev = false;
	bibbleData->m_bof.m_getOptionBool( pipeData->m_options, NP_ENABLED, bibbleData->m_plugin, enabled);
	return enabled;
}
